@@ -159,13 +159,13 @@ videoPtrToDriverList(struct pci_device *dev,
 	/* older Geode products acquired by AMD still carry an NSC vendor_id */
 	case 0x100b:
 	    if (dev->device_id == 0x0030) {
-		/* NSC Geode GX2 specifically ... */
+		/* NSC Geode GX2 specifically */
 		driverList[0] = "geode";
 		/* GX2 support started its life in the NSC tree and was later 
 		   forked by AMD for GEODE so we keep it as a backup */
 		driverList[1] = "nsc";
 	    } else 
-		/* ... any other NSC Geode SC variant e.g. 0x0504 for SCx200 */
+		/* other NSC variant e.g. 0x0104 (SC1400), 0x0504 (SCx200) */
 		driverList[0] = "nsc";
 	    break;
 	/* Cyrix Geode GX1 */
@@ -194,6 +194,7 @@ videoPtrToDriverList(struct pci_device *dev,
 	case 0x10c8:		    driverList[0] = "neomagic"; break;
 	case 0x10de: case 0x12d2:   driverList[0] = "nv";	break;
 	case 0x1106:		    driverList[0] = "openchrome"; break;
+        case 0x1b36:		    driverList[0] = "qxl"; break;
 	case 0x1163:		    driverList[0] = "rendition"; break;
 	case 0x5333:
 	    switch (dev->device_id)
@@ -272,7 +273,8 @@ xf86AutoConfig(void)
     for (cp = builtinConfig; *cp; cp++)
 	xf86ErrorFVerb(3, "\t%s", *cp);
     xf86MsgVerb(X_DEFAULT, 3, "--- End of built-in configuration ---\n");
-    
+
+    xf86initConfigFiles();
     xf86setBuiltinConfig(builtinConfig);
     ret = xf86HandleConfigFile(TRUE);
     FreeConfig();
