@@ -22,7 +22,6 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
  */
-/* $XConsortium: lnx_video.c /main/9 1996/10/19 18:06:34 kaleb $ */
 
 #ifdef HAVE_XORG_CONFIG_H
 #include <xorg-config.h>
@@ -39,7 +38,6 @@
 #include "xf86Priv.h"
 #include "xf86_OSlib.h"
 #include "xf86OSpriv.h"
-#include "lnx.h"
 #ifdef __alpha__
 #include "shared/xf86Axp.h"
 #endif
@@ -62,7 +60,8 @@ static Bool ExtendedEnabled = FALSE;
 #elif !defined(__powerpc__) && \
       !defined(__mc68000__) && \
       !defined(__sparc__) && \
-      !defined(__mips__)
+      !defined(__mips__) && \
+      !defined(__arm__)
 
 /*
  * Due to conflicts with "compiler.h", don't rely on <sys/io.h> to declare
@@ -74,27 +73,9 @@ extern int iopl(int __level);
 #endif
 
 #ifdef __alpha__
-
-# ifdef LIBC_IS_FIXED
-extern void sethae(unsigned long hae);
-# else
-#  include <unistd.h>
-#  define sethae(x) syscall(301,x);
-# endif
-
-/* define to test the Sparse addressing on a non-Jensen */
-# ifdef TEST_JENSEN_CODE 
-#  define isJensen (1)
-# else
-#  define isJensen (axpSystem == JENSEN)
-# endif
-
 # define BUS_BASE bus_base
-
 #else 
-
 #define BUS_BASE (0)
-
 #endif /*  __alpha__ */
 
 /***************************************************************************/
@@ -104,19 +85,18 @@ extern void sethae(unsigned long hae);
 static pointer mapVidMem(int, unsigned long, unsigned long, int);
 static void unmapVidMem(int, pointer, unsigned long);
 #if defined (__alpha__) 
+extern void sethae(unsigned long hae);
+extern unsigned long _bus_base __P ((void)) __attribute__ ((const));
+extern unsigned long _bus_base_sparse __P ((void)) __attribute__ ((const));
+
 static pointer mapVidMemSparse(int, unsigned long, unsigned long, int);
 extern axpDevice lnxGetAXP(void);
 static void unmapVidMemSparse(int, pointer, unsigned long);
-# if defined(JENSEN_SUPPORT)
-static pointer mapVidMemJensen(int, unsigned long, unsigned long, int);
-static void unmapVidMemJensen(int, pointer, unsigned long);
-# endif
 static axpDevice axpSystem = -1;
 static Bool needSparse;
 static unsigned long hae_thresh;
 static unsigned long hae_mask;
 static unsigned long bus_base;
-static unsigned long sparse_size;
 #endif
 
 #ifdef HAS_MTRR_SUPPORT
@@ -142,17 +122,8 @@ mtrr_open(int verbosity)
 	/* Only report absence of /proc/mtrr once. */
 	static Bool warned = FALSE;
 
-	char **fn;
-	static char *mtrr_files[] = {
-		"/dev/cpu/mtrr",	/* Possible future name */
-		"/proc/mtrr",		/* Current name */
-		NULL
-	};
-
 	if (mtrr_fd == MTRR_FD_UNOPENED) { 
-		/* So open it. */
-		for (fn = mtrr_files; mtrr_fd < 0 && *fn; fn++)
-			mtrr_fd = open(*fn, O_WRONLY);
+		mtrr_fd = open("/proc/mtrr", O_WRONLY);
 
 		if (mtrr_fd < 0)
 			mtrr_fd = MTRR_FD_PROBLEM;
@@ -317,9 +288,7 @@ mtrr_add_wc_region(int screenNum, unsigned long base, unsigned long size,
 		 lbase = lbase >> 1, d_size <<= 1);
 	    while (d_size > n_size)
 		d_size = d_size >> 1;
-#ifdef DEBUG
-	    ErrorF("WC_BASE: 0x%lx WC_END: 0x%lx\n",base,base+d_size-1);
-#endif
+	    DebugF("WC_BASE: 0x%lx WC_END: 0x%lx\n",base,base+d_size-1);
 	    n_base += d_size;
 	    n_size -= d_size;
 	    if (n_size) {
@@ -363,7 +332,7 @@ mtrr_undo_wc_region(int screenNum, struct mtrr_wc_region *wcr)
 {
 	struct mtrr_wc_region *p, *prev;
 
-	if (mtrr_fd > 0) {
+	if (mtrr_fd >= 0) {
 		p = wcr;
 		while (p) {
 			if (p->added)
@@ -403,21 +372,10 @@ xf86OSInitVidMem(VidMemInfoPtr pVidMem)
 	  if ((needSparse = (_bus_base_sparse() > 0))) {
 	    hae_thresh = xf86AXPParams[axpSystem].hae_thresh;
 	    hae_mask = xf86AXPParams[axpSystem].hae_mask;
-	    sparse_size = xf86AXPParams[axpSystem].size;
 	  }
 	  bus_base = _bus_base();
 	}
-	if (isJensen) {
-# ifndef JENSEN_SUPPORT
-	  FatalError("Jensen is not supported any more\n"
-		     "If you are intereseted in fixing Jensen support\n"
-		     "please contact xfree86@xfree86.org\n");
-# else
-	  xf86Msg(X_INFO,"Machine type is Jensen\n");
-	  pVidMem->mapMem = mapVidMemJensen;
-	  pVidMem->unmapMem = unmapVidMemJensen;
-# endif /* JENSEN_SUPPORT */
-	} else if (needSparse) {
+	if (needSparse) {
 	  xf86Msg(X_INFO,"Machine needs sparse mapping\n");
 	  pVidMem->mapMem = mapVidMemSparse;
 	  pVidMem->unmapMem = unmapVidMemSparse;
@@ -459,12 +417,10 @@ mapVidMem(int ScreenNum, unsigned long Base, unsigned long Size, int flags)
 
     realBase = Base & ~(getpagesize() - 1);
     alignOff = Base - realBase;
-#ifdef DEBUG
-    ErrorF("base: %lx, realBase: %lx, alignOff: %lx \n",
+    DebugF("base: %lx, realBase: %lx, alignOff: %lx \n",
 	   Base,realBase,alignOff);
-#endif
-    
-#if defined(__ia64__)
+
+#if defined(__ia64__) || defined(__arm__) || defined(__s390__)
 #ifndef MAP_WRITECOMBINED
 #define MAP_WRITECOMBINED 0x00010000
 #endif
@@ -504,9 +460,7 @@ mapVidMem(int ScreenNum, unsigned long Base, unsigned long Size, int flags)
 		   " (0x%08lx,0x%lx) (%s)\n", Base, Size,
 		   strerror(errno));
     }
-#ifdef DEBUG
-    ErrorF("base: %lx aligned base: %lx\n",base, base + alignOff);
-#endif
+    DebugF("base: %lx aligned base: %lx\n",base, base + alignOff);
     return (char *)base + alignOff;
 }
 #endif /* !(__sparc__) */
@@ -516,10 +470,8 @@ unmapVidMem(int ScreenNum, pointer Base, unsigned long Size)
 {
     memType alignOff = (memType)Base 
 	- ((memType)Base & ~(getpagesize() - 1));
-    
-#ifdef DEBUG
-    ErrorF("alignment offset: %lx\n",alignOff);
-#endif
+
+    DebugF("alignment offset: %lx\n",alignOff);
     munmap((caddr_t)((memType)Base - alignOff), (Size + alignOff));
 }
 
@@ -529,7 +481,7 @@ unmapVidMem(int ScreenNum, pointer Base, unsigned long Size)
 /***************************************************************************/
 
 #if defined(__powerpc__)
-_X_EXPORT volatile unsigned char *ioBase = NULL;
+volatile unsigned char *ioBase = NULL;
 
 #ifndef __NR_pciconfig_iobase
 #define __NR_pciconfig_iobase	200
@@ -537,7 +489,7 @@ _X_EXPORT volatile unsigned char *ioBase = NULL;
 
 #endif
 
-_X_EXPORT Bool
+Bool
 xf86EnableIO(void)
 {
 #if defined(__powerpc__)
@@ -567,7 +519,7 @@ xf86EnableIO(void)
 #endif
 	}
 	close(fd);
-#elif !defined(__mc68000__) && !defined(__sparc__) && !defined(__mips__) && !defined(__sh__) && !defined(__hppa__)
+#elif !defined(__mc68000__) && !defined(__sparc__) && !defined(__mips__) && !defined(__sh__) && !defined(__hppa__) && !defined(__s390__) && !defined(__arm__) && !defined(__m32r__)
         if (ioperm(0, 1024, 1) || iopl(3)) {
                 if (errno == ENODEV)
                         ErrorF("xf86EnableIOPorts: no I/O ports found\n");
@@ -586,7 +538,7 @@ xf86EnableIO(void)
 	return TRUE;
 }
 
-_X_EXPORT void
+void
 xf86DisableIO(void)
 {
 	if (!ExtendedEnabled)
@@ -594,7 +546,7 @@ xf86DisableIO(void)
 #if defined(__powerpc__)
 	munmap(ioBase, 0x20000);
 	ioBase = NULL;
-#elif !defined(__mc68000__) && !defined(__sparc__) && !defined(__mips__) && !defined(__sh__) && !defined(__hppa__)
+#elif !defined(__mc68000__) && !defined(__sparc__) && !defined(__mips__) && !defined(__sh__) && !defined(__hppa__) && !defined(__arm__) && !defined(__s390__) && !defined(__m32r__)
 	iopl(0);
 	ioperm(0, 1024, 0);
 #endif
@@ -603,76 +555,6 @@ xf86DisableIO(void)
 	return;
 }
 
-
-/***************************************************************************/
-/* Interrupt Handling section                                              */
-/***************************************************************************/
-
-/* XXX The #ifdefs should be made simpler. */
-
-_X_EXPORT Bool
-xf86DisableInterrupts()
-{
-#if !defined(__mc68000__) && !defined(__powerpc__) && !defined(__sparc__) && !defined(__mips__) && !defined(__ia64__) && !defined(__sh__) && !defined(__hppa__)
-	if (!ExtendedEnabled)
-	    if (iopl(3) || ioperm(0, 1024, 1))
-			return (FALSE);
-#endif
-#if defined(__alpha__) || defined(__mc68000__) || defined(__powerpc__) || defined(__sparc__) || defined(__mips__) || defined(__arm__) || defined(__sh__) || defined(__ia64__) || defined(__hppa__)
-#else
-# ifdef __GNUC__
-#  if defined(__ia64__)
-#   if 0
-	__asm__ __volatile__ (";; rsm psr.i;; srlz.d" ::: "memory");
-#   endif
-#  else
-      __asm__ __volatile__("cli");
-#  endif
-# else
-	asm("cli");
-# endif
-#endif
-#if !defined(__mc68000__) && !defined(__powerpc__) && !defined(__sparc__) && !defined(__mips__) && !defined(__sh__) && !defined(__ia64__) && !defined(__hppa__)
-	if (!ExtendedEnabled) {
-	    iopl(0);
-	    ioperm(0, 1024, 0);
-	}
-	
-#endif
-	return (TRUE);
-}
-
-_X_EXPORT void
-xf86EnableInterrupts()
-{
-#if !defined(__mc68000__) && !defined(__powerpc__) && !defined(__sparc__) && !defined(__mips__) && !defined(__ia64__) && !defined(__sh__) && !defined(__hppa__)
-	if (!ExtendedEnabled)
-	    if (iopl(3) || ioperm(0, 1024, 1))
-			return;
-#endif
-#if defined(__alpha__) || defined(__mc68000__) || defined(__powerpc__) || defined(__sparc__) || defined(__mips__) || defined(__arm__) || defined(__sh__) || defined(__ia64__) || defined(__hppa__)
-#else
-# ifdef __GNUC__
-#  if defined(__ia64__)
-#   if 0
-	__asm__ __volatile__ (";; ssm psr.i;; srlz.d" ::: "memory");
-#   endif
-#  else
-      __asm__ __volatile__("sti");
-#  endif
-# else
-	asm("sti");
-# endif
-#endif
-#if !defined(__mc68000__) && !defined(__powerpc__) && !defined(__sparc__) && !defined(__mips__) && !defined(__sh__) && !defined(__ia64__) && !defined(__hppa__)
-	if (!ExtendedEnabled) {
-	    iopl(0);
-	    ioperm(0, 1024, 0);
-	}
-#endif
-	return;
-}
-
 #if defined (__alpha__)
 
 #define vuip    volatile unsigned int *
@@ -974,194 +856,23 @@ writeSparseNB32(int Value, pointer Base, register unsigned long Offset)
     return;
 }
 
-_X_EXPORT void (*xf86WriteMmio8)(int Value, pointer Base, unsigned long Offset) 
+void (*xf86WriteMmio8)(int Value, pointer Base, unsigned long Offset)
      = writeDense8;
-_X_EXPORT void (*xf86WriteMmio16)(int Value, pointer Base, unsigned long Offset)
+void (*xf86WriteMmio16)(int Value, pointer Base, unsigned long Offset)
      = writeDense16;
-_X_EXPORT void (*xf86WriteMmio32)(int Value, pointer Base, unsigned long Offset)
+void (*xf86WriteMmio32)(int Value, pointer Base, unsigned long Offset)
      = writeDense32;
-_X_EXPORT void (*xf86WriteMmioNB8)(int Value, pointer Base, unsigned long Offset) 
+void (*xf86WriteMmioNB8)(int Value, pointer Base, unsigned long Offset)
      = writeDenseNB8;
-_X_EXPORT void (*xf86WriteMmioNB16)(int Value, pointer Base, unsigned long Offset)
+void (*xf86WriteMmioNB16)(int Value, pointer Base, unsigned long Offset)
      = writeDenseNB16;
-_X_EXPORT void (*xf86WriteMmioNB32)(int Value, pointer Base, unsigned long Offset)
+void (*xf86WriteMmioNB32)(int Value, pointer Base, unsigned long Offset)
      = writeDenseNB32;
-_X_EXPORT int  (*xf86ReadMmio8)(pointer Base, unsigned long Offset) 
+int  (*xf86ReadMmio8)(pointer Base, unsigned long Offset)
      = readDense8;
-_X_EXPORT int  (*xf86ReadMmio16)(pointer Base, unsigned long Offset)
+int  (*xf86ReadMmio16)(pointer Base, unsigned long Offset)
      = readDense16;
-_X_EXPORT int  (*xf86ReadMmio32)(pointer Base, unsigned long Offset)
+int  (*xf86ReadMmio32)(pointer Base, unsigned long Offset)
      = readDense32;
 
-#ifdef JENSEN_SUPPORT
-
-static int
-readSparseJensen8(pointer Base, register unsigned long Offset);
-static int
-readSparseJensen16(pointer Base, register unsigned long Offset);
-static int
-readSparseJensen32(pointer Base, register unsigned long Offset);
-static void
-writeSparseJensen8(int Value, pointer Base, register unsigned long Offset);
-static void
-writeSparseJensen16(int Value, pointer Base, register unsigned long Offset);
-static void
-writeSparseJensen32(int Value, pointer Base, register unsigned long Offset);
-static void
-writeSparseJensenNB8(int Value, pointer Base, register unsigned long Offset);
-static void
-writeSparseJensenNB16(int Value, pointer Base, register unsigned long Offset);
-static void
-writeSparseJensenNB32(int Value, pointer Base, register unsigned long Offset);
-
-/*
- * The Jensen lacks dense memory, thus we have to address the bus via
- * the sparse addressing scheme.
- *
- * Martin Ostermann (ost@comnets.rwth-aachen.de) - Apr.-Sep. 1996
- */
-
-#ifdef TEST_JENSEN_CODE 
-#define SPARSE (5)
-#else
-#define SPARSE (7)
-#endif
-
-#define JENSEN_SHIFT(x) ((long)x<<SPARSE)
-
-static pointer
-mapVidMemJensen(int ScreenNum, unsigned long Base, unsigned long Size, int flags)
-{
-  pointer base;
-  int fd, prot;
-
-  xf86WriteMmio8 = writeSparseJensen8;
-  xf86WriteMmio16 = writeSparseJensen16;
-  xf86WriteMmio32 = writeSparseJensen32;
-  xf86WriteMmioNB8 = writeSparseJensenNB8;
-  xf86WriteMmioNB16 = writeSparseJensenNB16;
-  xf86WriteMmioNB32 = writeSparseJensenNB32;
-  xf86ReadMmio8 = readSparseJensen8;
-  xf86ReadMmio16 = readSparseJensen16;
-  xf86ReadMmio32 = readSparseJensen32;
-
-  fd = open(DEV_MEM, (flags & VIDMEM_READONLY) ? O_RDONLY : O_RDWR);
-  if (fd < 0) {
-    FatalError("xf86MapVidMem: failed to open " DEV_MEM " (%s)\n",
-	       strerror(errno));
-  }
-
-  if (flags & VIDMEM_READONLY)
-    prot = PROT_READ;
-  else
-    prot = PROT_READ | PROT_WRITE;
-
-  /* This requires linux-0.99.pl10 or above */
-  base = mmap((caddr_t)0, JENSEN_SHIFT(Size),
-	      prot, MAP_SHARED, fd,
-	      (off_t)(JENSEN_SHIFT((off_t)Base) + _bus_base_sparse()));
-  close(fd);
-  if (base == MAP_FAILED) {
-    FatalError("xf86MapVidMem: Could not mmap framebuffer"
-	       " (0x%08x,0x%x) (%s)\n", Base, Size,
-	       strerror(errno));
-  }
-  return base;
-}
-
-static void
-unmapVidMemJensen(int ScreenNum, pointer Base, unsigned long Size)
-{
-  munmap((caddr_t)Base, JENSEN_SHIFT(Size));
-}
-
-static int
-readSparseJensen8(pointer Base, register unsigned long Offset)
-{
-    register unsigned long result, shift;
-
-    mem_barrier();
-    shift = (Offset & 0x3) << 3;
-
-    result = *(vuip) ((unsigned long)Base + (Offset << SPARSE));
-
-    result >>= shift;
-    return 0xffUL & result;
-}
-
-static int
-readSparseJensen16(pointer Base, register unsigned long Offset)
-{
-    register unsigned long result, shift;
-
-    mem_barrier();
-    shift = (Offset & 0x2) << 3;
-
-    result = *(vuip)((unsigned long)Base+(Offset<<SPARSE)+(1<<(SPARSE-2)));
-
-    result >>= shift;
-    return 0xffffUL & result;
-}
-
-static int
-readSparseJensen32(pointer Base, register unsigned long Offset)
-{
-    register unsigned long result;
-
-    mem_barrier();
-    result = *(vuip)((unsigned long)Base+(Offset<<SPARSE)+(3<<(SPARSE-2)));
-
-    return result;
-}
-
-static void
-writeSparseJensen8(int Value, pointer Base, register unsigned long Offset)
-{
-    register unsigned int b = Value & 0xffU;
-
-    write_mem_barrier();
-    *(vuip) ((unsigned long)Base + (Offset << SPARSE)) = b * 0x01010101;
-}
-
-static void
-writeSparseJensen16(int Value, pointer Base, register unsigned long Offset)
-{
-    register unsigned int w = Value & 0xffffU;
-
-    write_mem_barrier();
-    *(vuip)((unsigned long)Base+(Offset<<SPARSE)+(1<<(SPARSE-2))) =
-      w * 0x00010001;
-}
-
-static void
-writeSparseJensen32(int Value, pointer Base, register unsigned long Offset)
-{
-    write_mem_barrier();
-    *(vuip)((unsigned long)Base+(Offset<<SPARSE)+(3<<(SPARSE-2))) = Value;
-}
-
-static void
-writeSparseJensenNB8(int Value, pointer Base, register unsigned long Offset)
-{
-    register unsigned int b = Value & 0xffU;
-
-    *(vuip) ((unsigned long)Base + (Offset << SPARSE)) = b * 0x01010101;
-}
-
-static void
-writeSparseJensenNB16(int Value, pointer Base, register unsigned long Offset)
-{
-    register unsigned int w = Value & 0xffffU;
-
-    *(vuip)((unsigned long)Base+(Offset<<SPARSE)+(1<<(SPARSE-2))) =
-      w * 0x00010001;
-}
-
-static void
-writeSparseJensenNB32(int Value, pointer Base, register unsigned long Offset)
-{
-    *(vuip)((unsigned long)Base+(Offset<<SPARSE)+(3<<(SPARSE-2))) = Value;
-}
-#endif /* JENSEN_SUPPORT */
-
 #endif /* __alpha__ */
